/***************************************************/
/*  Mootools: Nested Sortable Lists               */
/*  Author: Simon Ilett                          */
/*  Version: 1.3								*/
/*  Date: 29/07/2011 						   */
/**********************************************/

var Nested = new Class({

	Implements: [Options, Events],
	options: {
		childTag: 'LI',
		ghost: true,
		collapse: true,
		childStep: 40, // attempts to become a child if the mouse is moved this number of pixels right
		closedSpace: 3, // space for top and bottom insert detection for closed nodes
		onStart: $empty,
		onComplete: $empty,
		lockClass: 'locked'
	},


	initialize: function(list, options) {
		this.setOptions(options);
		
		// Root list
		this.list = $(list);
		this.options.parentTag = this.list.nodeName;
		
		// Object co-ords and events
		this.bound = {};
		this.bound.start = this.start.bindWithEvent(this); 
		this.list.addEvent('mousedown', this.bound.start); 
		
		/* Extra options stuff */
		if (this.options.collapse) this.setToggle('add');
		if (this.options.check) this.setCheck();
		
	},

	start: function(event) {
		
		// Find our target and initialise
		event.preventDefault();
		var el = $(event.target);
		if(!this.isTargetAllowed(el)) return;
		
		/* bubble up to the parent LI!!! but avoid the root node */
		while (el.nodeName != this.options.childTag && el != this.list) {
			el = el.parentNode;
		}
		// If conditions for draggable row
		if(!this.isRowAllowed(el)) return;
		
		// Create the ghost
		if (this.options.ghost) 
			this.ghost = this.createGhost(el, event).injectInside(document.body);
		
		// Important attributes for the row
		//el.depth = this.getDepth(el);
		el.moved = false;
		
		// All the events
		this.bound.movement = this.movement.bindWithEvent(this, el);
		this.bound.end = this.end.bind(this, el);
		this.list.addEvent('mousemove', this.bound.movement);
		document.addEvent('mouseup', this.bound.end);
		this.fireEvent('onStart', el);
		
	},

	
	
	
	movement: function(event, el) {
		var dir, over, check, items, dest, move, prev, prevParent, sub, last,
		abort = false,
		pageY = event.page.y,
		pageX = event.page.x;
		
		// Position the ghost
		if (this.options.ghost)  this.positionGhost(pageX, pageY);
		
		// bubble up to the ROW that is hovered over
		over = event.target;
		if (over == this.list) return;
		while (over.nodeName != this.options.childTag && over != this.list) {
			over = over.parentNode;
		}
		// Check if it's actually inline with a child element of the event firer
		if (el != over) {
			items = over.getElements(this.options.childTag); 
			items.each(function(item) {
				if (pageY > item.getTop() && item.offsetHeight > 0) over = item;
			});
		}
			
		// store the previous parent 'ul' to remove it if a move makes it empty
		prevParent = el.getParent();
		// directory path based off mouse vert direction
		dir = (pageY < el.getTop() ? 'up' : 'down');
		// inject switch default 
		move = 'before';
		// destination defaulting to the selected row (will cause abort)
		dest = el;
		
		// If over a row that is not the selected drag row
		if (el != over) {
			/*
			Make sure we're not trying to move row below/into itself
		 	Bubble up the tree of the over row and make sure its not a child of the parent drag row.
		 	if check is not the selected row then its valid
			*/
			check = over;
			while (check != null && check != el) {
				check = check.parentNode;
			} 
			if (check == el) return;
			
			// Build our inject statement
			if (dir == 'up') {
				move = 'before'; dest = over;
			} else {
				sub = over.getElement(this.options.childTag); 
				//if children and child has height (height for closed nodes)
				if (sub && sub.offsetHeight > 0) {
					//move before the first child of the over row
					move = 'before'; dest = sub;
				} else {
					//move after the over row
					move = 'after'; dest = over;
				}
			}
		}
		
	
		// Check if we're trying to go deeper -->>
		prev = (move == 'before' ? dest.getPrevious() : dest);
		if (prev) {
			move = 'after';
			dest = prev;
			check = dest.getElement(this.options.parentTag);
			while (check && pageX > check.getLeft() && check.offsetHeight > 0) {
				dest = check.getLast();
				check = dest.getElement(this.options.parentTag);
			}
			if (!check && pageX > dest.getLeft()+this.options.childStep) {
				move = 'inside';
			}
		}
		
		// Get the last row of the destination
		last = dest.getParent().getLast();
		while (((move == 'after' && last == dest) || last == el) && dest.getParent() != this.list && pageX < dest.getLeft()) {
			move = 'after';
			dest = $(dest.parentNode.parentNode);
			last = dest.getParent().getLast();
		}
		
		// Check row and ancestors for a lock then action it.
		this.lockedNode(this.isLocked(move, dest, last), el);	
		
		
		// Do the move and final checks
		abort = false;
		if (move != '') {
			abort += (dest == el);
			abort += (dest.offsetHeight == 0);
			abort += (this.lock);
			sub = over.getElement(this.options.parentTag);
			sub = (sub) ? sub.getTop() : 0;
			sub = (sub > 0) ? sub-over.getTop() : over.offsetHeight;
			abort += (pageY < (sub-el.offsetHeight)+over.getTop());
			if (!abort) {
				
				// Over row is valid target but closed
				this.closedNode(over, el);
				// Allows position of nodes between closed nodes
				if (this.over) {
					if (dir=='up' && pageY-this.options.closedSpace < this.over.getTop())
						this.over.fireEvent('mouseleave', this);
					if (dir=='down' && pageY > this.over.getTop()+(over.offsetHeight-this.options.closedSpace))	
						this.over.fireEvent('mouseleave', this);
				}
				// All other node positioning
				if (!this.over) {
					if (move == 'inside') dest = new Element(this.options.parentTag).injectInside(dest);
					$(el).inject(dest, move);
					if (!prevParent.getFirst()) prevParent.destroy();
				}
				el.moved = true;
					
			}
			
		}
		event.stop();
	},

	
	
	end: function(el) {
		// Was dropped on a closed row
		if (this.over) this.closedRow(el);
		// Remove the ghosting div
		if (this.options.ghost && this.ghost) this.ghost.destroy();
		// Refresh the toggles
		if (this.options.collapse) this.setToggle('refresh');
		// Remove events
		document.removeEvent('mouseup', this.bound.end);
		this.list.removeEvent('mousemove', this.bound.movement);
		// Completed
		this.fireEvent('onComplete', el);
	},
	
	/************/
	/* Helpers */
	
	
	// How far down is the node
	getDepth: function(el, add) {
		var counter = (add) ? 1 : 0;
		while (el != this.list) {
			if (el.nodeName == this.options.parentTag) counter += 1;
			el = el.parentNode;
		}
		return counter;
	},
	
	// Is row or any ancestor row locked
	isLocked: function(move, dest, last) {
		// Check if the row is locked 
		if (move == 'inside' && dest.hasClass('locked')) {
			var ancestor = dest;
		} else {	
			var ancestor = last;
			console.log(ancestor);
			while (ancestor != this.list && ancestor != this.ghost && !ancestor.hasClass('locked')) {
				ancestor = ancestor.parentNode;
			}
			
			// special case :(
			if (ancestor == this.list.getLast())  ancestor = this.list; 
		}		 
		return ancestor;
	},
	
	// Actions for hovering over locked row
	lockedNode: function(over, el) {
		this.showLock = (this.options.ghost ? this.ghost : el);
		if (over.hasClass('locked')) {		
			if (!this.lock) {
				this.lock = over;
				this.lock.addEvent('mouseleave', this.unsetLockStyle.bindWithEvent(this));
				this.setLockStyle();	
			}
		} else {
			if (this.lock) {
				this.unsetLockStyle();	
				this.lock = null;		
			}
		}	 
	},
	
	
	// Actions for hovering over closed row
	closedNode: function(over, el) {
		if (over.hasClass('remove') && !over.hasClass('over') && el != over && !over.hasClass('locked')) {
			this.setOverStyle(over);
			this.over = over;
			this.over.addEvent('mouseleave', this.unsetOverStyle.bindWithEvent(this));
		} 
	},
	
	// Actions for dropping over closed row
	closedRow: function(el) {
		// close children
		if (el.getElement('UL')) 
			el.getElements('UL').toggleClass('hide');
		var dest = this.over.getElement('UL');
		var saveParent = el.parentNode;
		$(el).inject(dest, 'top');	
		if (!saveParent.getChildren().length) {
			saveParent.destroy();
		} 
		this.unsetOverStyle();
	},
	
	// Create the ghost obj
	createGhost: function(el, e) {
		return el.clone().setStyles({
			'list-style-type': 'none',
			'opacity': 0.5,
			'position': 'absolute',
			'visibility': 'hidden',
			'top': e.page.y+'px',
			'left': (e.page.x+10)+'px'
		}).set('id', 'ghostNode');
	},
	
	// Position the ghost obj
	positionGhost: function(x, y) {
		this.ghost.setStyles({
			'visibility': 'visible',
			'top': y+'px',
			'left': (x+10)+'px'
		});
	},
	
	// Cases to exit drag (based off the target clicked in a row)
	isTargetAllowed: function (el) {
		var tmp = true;
		if (el.hasClass('toggle')) tmp = false;
		return tmp;
	},	
	
	// Cases to exit drag (based off the row)
	isRowAllowed: function (el) {
		var tmp = true;
		if (el.nodeName != this.options.childTag) tmp = false;
		var ancestor = el;
		while (ancestor != this.list && !ancestor.hasClass(this.options.lockClass)) {
			ancestor = ancestor.parentNode;
		}
		if (ancestor.hasClass(this.options.lockClass)) tmp = false;
		return tmp;
	},	
	
	
	/***************/
	/* row styles */
	
	onStyle: function(el) {
		/*
		this.tmp = new Element('DIV', {
			'class': 'line'
		});
		this.tmp.inject(el, 'top');
		el.addClass('lineOn');
		*/
		el.setStyle('opacity', '0.5');
		el.addClass('drag');
	},
	
	offStyle: function(el) {
		//this.tmp.destroy();
		//el.removeClass('lineOn');
		el.setStyle('opacity', '1');
		el.removeClass('drag');
	},
	
	setOverStyle: function(over) {
		over.addClass('over');
	},
	
	unsetOverStyle: function() {
		if (this.over)
			this.over.removeClass('over');
		this.over = null;
	},
	
	setLockStyle: function() {
		this.showLock.addClass('lock');
	},
	
	unsetLockStyle: function() {
		if (this.showLock) this.showLock.removeClass('lock');
	}
	
});




/* Plugins for Nested sortables */




/* TOGGLES */
/*  re-initialise the open/close icons */	

Nested.implement({
	
	setToggle: function(action){
		var self = this;
		$$('ul#example li').each(function(el) {
			self.toggleLogic(action, el);
		});
	},
	
	toggleLogic: function(tmp, el) {
		var self = this;
		if (tmp == 'add') {
			var toggle = new Element('div').setProperties({'class':'toggle'});
			toggle.inject(el, 'top');
		} else {
			var toggle = el.getElement('.toggle');
	 		toggle.removeEvents();
		}
	
		if (el.getElement('ul')) {
			toggle.addEvent('click', self.toggleRemove).removeClass('inactive');
		} else {
			toggle.addClass('inactive');
		}
		
	},
	
	toggleRemove: function() {
		this.parentNode.toggleClass('remove');
		this.parentNode.getElements('ul').toggleClass('hide');
	}
	
});


/* CHECKS */
/*  check box stuff (Can't remember why I wanted this) */	
Nested.implement({
	setCheck: function(){}
});



/* SAVING & TRANSPORT */	
Nested.implement({
	serialize: function(listEl){
		var serial = [];
		var kids;
		if (!listEl) listEl = this.list;
		$$(listEl.childNodes).each(function(node, i) {
			kids = node.getElement(this.options.parentTag);
			serial[i] = {
				id: node.id,
				action: node.getProperty('class'),
				title: node.title,
				children: (kids) ? this.serialize(kids) : []
			};
		}.bind(this));
		return serial;
	}
});
